/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation, either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/** Basic text widget and supporting code for widgets containing text.
 *
 * All content widgets have one (at least for basic content widgets) Content, from
 * mde.gui.content.Items . */
module mde.gui.widget.TextWidget;

import mde.gui.widget.Widget;
import mde.gui.exception;
import mde.gui.renderer.IRenderer;
import mde.gui.content.Content;

import mde.font.font;

/// Adapter to ease use of ContentLabelWidget
struct TextAdapter {
    void set (char[] c, int col) {
        //FIXME tie font to renderer or so
        if (font is null) font = FontStyle.get("default");
        
        content = c;
        colour = Colour (col);
    }
    
    void getDimensions (out wdsize w, out wdsize h) {
        font.updateBlock (content, textCache);
        w = cast(wdim) textCache.w;
        h = cast(wdim) textCache.h;
    }
    
    void draw (wdabs x, wdabs y) {
        font.textBlock (x,y, content, textCache, colour);
    }
    
    char[] content;
    TextBlock textCache;
    Colour colour;
    static FontStyle font;
}

/// Basic text widget
class TextLabelWidget : Widget
{
    /** Constructor for a widget containing [fixed] content.
     *
     * Widget uses the initialisation data:
     * [widgetID, contentID, colour]
     * where contentID is an ID for the string ID of the contained content
     * and colour is an 8-bit-per-channel RGB colour of the form 0xRRGGBB. */
    this (IWidgetManager mgr, widgetID id, WidgetData data) {
        WDCheck (data, 2, 1);
        adapter.set (data.strings[0], data.ints[1]);
        adapter.getDimensions (mw, mh);
        super (mgr, id, data);
    }
    
    void draw () {
        super.draw();
        adapter.draw (x,y);
    }
    
protected:
    TextAdapter adapter;
}

/// Basic widget displaying a label from a content.
class ContentLabelWidget : Widget
{
    this (IWidgetManager mgr, widgetID id, WidgetData data, IContent c) {
        WDCheck (data, 3, 0);
        content = c;
        index = data.ints[1];
        adapter.set (content.toString(index), data.ints[2]);
        adapter.getDimensions (mw, mh);
        super (mgr, id,data);
    }
    
    void draw () {
        super.draw();
        adapter.draw (x,y);
    }
    
protected:
    TextAdapter adapter;
    IContent content;
    int index;
}
